!> \file climat_InitMIP_mod.f90
! forcage avec BM
!! Module ou les variations temporelles des variables climatiques
!! sont directement imposees
!<

!> \namespace  climat_InitMIP_mod
!! Module ou les variations temporelles des variables climatiques
!! sont directement imposees en suivant le protocol initMIP
!! \author afq
!! \date 10 oct 17
!! @note Used modules:
!! @note   - use module3D_phy
!<
module climat_InitMIP_years_perturb_mod


use module3d_phy,only: nx,ny,S,S0,H,Bsoc,acc,abl,BM,Tann,Tjuly,Ts,time,dt,num_param,num_rep_42,num_forc,dirforcage,dirnameinp,tafor,time,sealevel,sealevel_2d,coefbmshelf
use netcdf
use io_netcdf_grisli
implicit none


real :: T_lapse_rate              !< pour la temperature

! anomalies: smb and Tann
real,dimension(:,:,:),allocatable   :: smb_anom    !> bilan de masse des anomalies indices : nx,ny,tsnap
real,dimension(:,:,:),allocatable   :: tann_anom   !> Tann anomalies indices : nx,ny,tsnap
real,dimension(:),allocatable       :: time_snap   !> date des snapshots  indice : nb de nb_snap
real,dimension(:,:,:),allocatable   :: grad_smb     !> gradient vertical de smb indices : nx,ny,tsnap
real,dimension(:,:,:),allocatable   :: grad_tann   !> gradient vertical de smb indices : nx,ny,tsnap

! declaration pour le bilan de masse et le bmelt
real,dimension(nx,ny)    :: bm_0              !> bilan de masse initial

real                     :: coef_smb_unit     ! pour corriger l'unite
real                     :: coef_smb_anom_unit ! facteur correction unit anomalies de smb
real,dimension(nx,ny)    :: TA0               !> Temp annuelle sur S0


! aurel, pour climat type perturb:
!integer nft             !> nombre de lignes a lire dans le fichier forcage climatique
!real,dimension(:),allocatable :: tdate          !> time for climate forcing
!real,dimension(:),allocatable :: tpert          !> temperature for climate forcing
!real,dimension(:),allocatable :: spert          !> sea surface perturbation
!real :: coefT                    !> pour modifier l'amplitude de la perturb. T
!character(len=120) :: filforc    !> nom du fichier forcage
!integer :: pertsmb               !> boolean: do we modify the smb?
!real    :: rapsmb                !> if we modify the smb this is the equivalent of rappact


! pour les lectures ncdf
real*8, dimension(:), pointer        :: tab1D             !< tableau 1d real pointer
real*8, dimension(:,:),   pointer    :: tab2D             !< tableau 2d real pointer
real*8, dimension(:,:,:), pointer    :: tab3D             !< tableau 3d real pointer
Character(len=10), dimension(3)      :: dimtestname       !> pour la sortie test netcdf
integer                              :: ncidloc           !> pour la sortie test netcdf
integer                              :: status            !> pour la sortie test netcdf

integer                              :: massb_time        !< pour selectionner le type de calcul de smb

integer                              :: nb_snap           !> nombre de snapshots

real,dimension(nx,ny)                :: smb_anom_last20years  !> anomalie de smb des 20 dernieres annees du forcage
real,dimension(nx,ny)                :: tann_anom_last20years !> anomalie de Tann des 20 dernieres annees du forcage

contains

!--------------------------------------------------------------------------------
!> SUBROUTINE: input_clim  
!! Routine qui permet d'initialiser les variations temporelles des variables climatiques
!>
!--------------------------------------------------------------------------------

subroutine input_clim  

  character(len=100) :: smb_file           ! fichier smb
  character(len=100) :: temp_annual_file   ! temperature annuelles
  character(len=100) :: file_smb_anom      !> nom du fichier dans lequel il y a l'anomalie smb
  character(len=100) :: file_lapse         !> nom du fichier dans lequel il y a les lapserates (smb et tann)

  integer            :: err                ! recuperation d'erreur
  integer            :: i,k

  !aurel for Tafor:
  character(len=8)   :: control      !label to check clim. forc. file (filin) is usable
  character(len=80)  :: filin
  
  real               :: time_depart_snaps  !> temps du debut premier snapshot

  namelist/clim_smb_T_gen/smb_file,coef_smb_unit,temp_annual_file
! version Aurelien Groenland avec file_lapse :
!  namelist/smb_anom_initMIP/file_smb_anom,file_lapse,coef_smb_anom_unit,nb_snap,time_depart_snaps,massb_time
! version utilisee pour Antarctique :  
  namelist/smb_anom_initMIP/file_smb_anom,file_lapse,coef_smb_anom_unit,nb_snap,time_depart_snaps,massb_time
  
!!!!!  namelist/clim_snap/nb_snap,file_smb_snap,massb_time_snap

428 format(A)
  rewind(num_param)                     ! pour revenir au debut du fichier param_list.dat
  read(num_param,clim_smb_T_gen)

  write(num_rep_42,428)'!________________________________________________________________' 
  write(num_rep_42,428)'!  module climat_InitMIP_years_perturb_mod lecture climat ref          '
  write(num_rep_42,clim_smb_T_gen)
  write(num_rep_42,428)'! smb_file          = fichier SMB (kg/m2/an)                     '
  write(num_rep_42,428)'! coef_smb_unit     = coef passage m glace/an  (1/910 ou 1/918)  '
  write(num_rep_42,428)'! temp_annual_file  = Temp moy annuelle  (C)                    '
  write(num_rep_42,428)'!________________________________________________________________'

  
! smb : surface mass balance
  smb_file  = trim(dirnameinp)//trim(smb_file)

  call Read_Ncdf_var('smb',smb_file,tab2D)


  bm(:,:)  = tab2D(:,:) * coef_smb_unit

  !where ((H(:,:).lt.1.))
  !   bm(:,:) = bm(:,:) - 15.                     ! pour faire un masque a l'exterieur du Groenland actuel
  !end where

  acc(:,:) = 0.
  abl(:,:) = 0.

  where (bm(:,:).gt.0.)
     acc(:,:) = bm(:,:)   ! accumulation quand positif
  elsewhere
     abl(:,:) = - bm(:,:) ! ablation quand negatif
  end where


! surface temperature  Tann

  temp_annual_file = trim(dirnameinp)//trim(temp_annual_file)

  call Read_Ncdf_var('Tann',temp_annual_file,tab2D)
  Tann(:,:)  = tab2D(:,:)

  ta0(:,:)   = Tann(:,:)
  Tjuly(:,:) = Tann(:,:)
  bm_0(:,:) = bm(:,:)

  sealevel=0.
  tafor=0.
  sealevel_2d(:,:) = sealevel

  ! ______ Anomalies de SMB (simule asmb initMIP) 
  rewind(num_param)        ! pour revenir au debut du fichier param_list.dat
  read(num_param,smb_anom_initMIP)

  write(num_rep_42,428)'!____________________________________________________________________________' 
  write(num_rep_42,428)'!  module climat_InitMIP_years_perturb_mod                                   '
  write(num_rep_42,smb_anom_initMIP)
  write(num_rep_42,428)'! file_smb_anom = fichier anomalie SMB de GCM                                '
  write(num_rep_42,428)'! file_lapse    = fichier des lapse rate SMB et Tann                         '
  write(num_rep_42,428)'! coef_smb_anom_unit = coef passage m glace/an  (1/910 ou 1/918)             '
  write(num_rep_42,428)'! nb_snap       = nombre de snapshots                                        '
  write(num_rep_42,428)'! time_depart_snaps = debut du forage                                       '
  write(num_rep_42,428)'! massb_time = 0:fixe, 1:anomalies, 2:interp snapshots, 3:snapsh+interp vert '
  write(num_rep_42,428)'!____________________________________________________________________________'
  
  file_smb_anom  = trim(dirnameinp)//trim(file_smb_anom)
  file_lapse     = trim(dirnameinp)//trim(file_lapse)

  if (massb_time == 1) then ! fichier 2D de smb (exp initMIP asmb)
! allocation dynamique de smb_snap
      if (allocated(smb_anom)) then 
        deallocate(smb_anom,stat=err)
        if (err/=0) then
          print *,"Erreur  la desallocation de smb_anom",err
          stop 
        end if
      end if

      allocate(smb_anom(nx,ny,1),stat=err)
      if (err/=0) then
        print *,"erreur a l'allocation du tableau smb_anom ",err
        print *,"nx,ny,1 = ",nx,',',ny,',',1
        stop 
      end if
      call Read_Ncdf_var('asmb',file_smb_anom,tab2D)
      smb_anom(:,:,1) = Tab2D(:,:) * coef_smb_anom_unit
  elseif (massb_time >= 2) then ! fichier 3D de smb : lecture des snapshots
! allocation dynamique de time_snap
      if (allocated(time_snap)) then 
        deallocate(time_snap,stat=err)
        if (err/=0) then
          print *,"Erreur  la desallocation de time_snap",err
          stop 
        end if
      end if

      allocate(time_snap(nb_snap),stat=err)
      if (err/=0) then
        print *,"erreur a l'allocation du tableau time_snap ",err
        print *,"nb_snap = ",nb_snap
        stop 
      end if
! allocation dynamique de smb_snap
      if (allocated(smb_anom)) then 
        deallocate(smb_anom,stat=err)
        if (err/=0) then
          print *,"Erreur  la desallocation de smb_anom",err
          stop 
        end if
      end if

      allocate(smb_anom(nx,ny,nb_snap),stat=err)
      if (err/=0) then
        print *,"erreur a l'allocation du tableau smb_anom ",err
        print *,"nx,ny,nb_snap = ",nx,',',ny,',',nb_snap
        stop 
      end if

! allocation dynamique de tann_snap
      if (allocated(tann_anom)) then 
        deallocate(tann_anom,stat=err)
        if (err/=0) then
          print *,"Erreur  la desallocation de tann_anom",err
          stop 
        end if
      end if

      allocate(tann_anom(nx,ny,nb_snap),stat=err)
      if (err/=0) then
        print *,"erreur a l'allocation du tableau tann_anom ",err
        print *,"nx,ny,nb_snap = ",nx,',',ny,',',nb_snap
        stop 
      end if
  
! lecture de smb_snap
      call Read_Ncdf_var('smb_anomaly',file_smb_anom,tab3D)
      smb_anom(:,:,:) = tab3D(:,:,:) * coef_smb_anom_unit
  
! lecture de tann_snap
      call Read_Ncdf_var('ts_anomaly',file_smb_anom,tab3D)
      tann_anom(:,:,:) = tab3D(:,:,:)

! lecture de time_snap
      call Read_Ncdf_var('time',file_smb_anom,tab1D)
      time_snap(:) = tab1D(:)
      time_snap(:) = time_snap(:) + time_depart_snaps
    
      if (massb_time == 3) then ! lecture lapse rate
         
         ! allocation dynamique de grad_smb
         if (allocated(grad_smb)) then 
            deallocate(grad_smb,stat=err)
            if (err/=0) then
               print *,"Erreur  la desallocation de grad_smb",err
               stop 
            end if
         end if

         allocate(grad_smb(nx,ny,nb_snap),stat=err)
         if (err/=0) then
            print *,"erreur a l'allocation du tableau grad_smb ",err
            print *,"nx,ny,nb_snap = ",nx,',',ny,',',nb_snap
            stop 
         end if

      
         ! allocation dynamique de smb_snap
         if (allocated(grad_tann)) then 
            deallocate(grad_tann,stat=err)
            if (err/=0) then
               print *,"Erreur  la desallocation de grad_tann",err
               stop 
            end if
         end if

         allocate(grad_tann(nx,ny,nb_snap),stat=err)
         if (err/=0) then
            print *,"erreur a l'allocation du tableau grad_tann ",err
            print *,"nx,ny,nb_snap = ",nx,',',ny,',',nb_snap
            stop 
         end if
     
         call Read_Ncdf_var('smb_gradient',file_lapse,tab3D)
         grad_smb(:,:,:) = tab3D(:,:,:) * coef_smb_anom_unit
         
         call Read_Ncdf_var('ts_gradient',file_lapse,tab3D)
         grad_tann(:,:,:) = tab3D(:,:,:)
         
      endif
! calcul de la moyenne des 20 dernieres annees du forcage
    do k=nb_snap,nb_snap-19,-1
      smb_anom_last20years(:,:) = smb_anom_last20years(:,:) + smb_anom(:,:,k)
      tann_anom_last20years(:,:) = tann_anom_last20years(:,:) + tann_anom(:,:,k)
    enddo
      smb_anom_last20years(:,:) = smb_anom_last20years(:,:)/20.
      tann_anom_last20years(:,:) = tann_anom_last20years(:,:)/20.
  endif     

   
end subroutine input_clim

!--------------------------------------------------------------------------------
!> SUBROUTINE: init_forclim
!! Routine qui permet d'initialiser les variables climatiques au cours du temps
!>
subroutine init_forclim

  implicit none
  
end subroutine init_forclim

!--------------------------------------------------------------------------------
!> SUBROUTINE: forclim
!!  
!!  Routine qui permet le calcul climatique au cours du temps
!>

subroutine forclim               !  au temps considere (time) 

  implicit none

  integer i,ift
  real             :: coefanomtime
  
  
  if (massb_time == 0) then
    bm(:,:) = bm_0(:,:)
  elseif (massb_time == 1) then
    coefanomtime = min ( real(time/40.) , 1. ) 
    bm(:,:) = bm_0(:,:) + coefanomtime * smb_anom(:,:,1)
  elseif (massb_time >= 2) then
    call massb_ISMIP_RCM
  end if
  
  Ts(:,:)    = Tann(:,:)

       
! coefmshelf est un coefficient qui fait varier bmgrz et bmshelf en fonction de tafor
  coefbmshelf=1.
      
end subroutine forclim

subroutine massb_ISMIP_RCM                ! calcule le mass balance 

  implicit none
  integer             :: k                ! pour calculer les indices de temps
  real,dimension(nx,ny) :: bm_time, tann_time ! pour calcul Bm et Tann
  real,dimension(nx,ny) :: grad_smb_time, grad_tann_time ! pour calcul Bm et Tann avec correction verticale

! calcul smb a partir fichier snapshots massb_ISMIP_RCM
! Calcule le mass balance d'apres un fichier de snapshots

! calcule bm_time par interpolation entre deux snapshots
! avant prend la valeur de reference
! apres prend la derniere valeur
! en general les snapshots vont de 1995 a 2100 (time_snap de 0  105)
  if(time.lt.time_snap(1)) then              ! time avant le forcage
     bm_time(:,:) = bm_0(:,:)
     tann_time(:,:) = ta0(:,:)
     if (massb_time == 3 ) then
        grad_smb_time(:,:) =   grad_smb (:,:,1)
        grad_tann_time(:,:) = grad_tann(:,:,1)
     endif
  else if (time.ge.time_snap(nb_snap)) then  ! time apres le forcage : constant = derniere anne nb_snap
     bm_time(:,:) =  bm_0(:,:) + smb_anom (:,:,nb_snap)
     tann_time(:,:) = ta0(:,:) + tann_anom(:,:,nb_snap)
     if (massb_time == 3 ) then
        grad_smb_time(:,:) =   grad_smb (:,:,nb_snap)
        grad_tann_time(:,:) = grad_tann(:,:,nb_snap)
     endif
!~   else if (time.ge.time_snap(nb_snap)) then  ! time apres le forcage : constant moy 20 dernieres annes 
!~      bm_time(:,:) =  bm_0(:,:) + smb_anom_last20years(:,:)
!~      tann_time(:,:) = ta0(:,:) + tann_anom_last20years(:,:)
  else                                       ! cas general
     do k = 1 , nb_snap-1
        if((time.ge.time_snap(k)).and.(time.lt.time_snap(k+1))) then ! entre k et k+1 
!cdc version avec interpolation lineaire entre 2 annes        
!~            bm_time(:,:) = bm_0(:,:) + smb_anom(:,:,k) + (smb_anom(:,:,k+1)-smb_anom(:,:,k)) *   &
!~                 (time-time_snap(k))/(time_snap(k+1)-time_snap(k))
!~            tann_time(:,:) = ta0(:,:) + tann_anom(:,:,k) + (tann_anom(:,:,k+1)-tann_anom(:,:,k)) *   &
!~                 (time-time_snap(k))/(time_snap(k+1)-time_snap(k))
!cdc version ISMIP : forcage annee n pendant un an sans interpolation.
           bm_time(:,:) = bm_0(:,:) + smb_anom(:,:,k)
           tann_time(:,:) = ta0(:,:) + tann_anom(:,:,k)
           if (massb_time == 3 ) then
              grad_smb_time(:,:) =   grad_smb(:,:,k)
              grad_tann_time(:,:) = grad_tann(:,:,k)
           endif
           exit
        endif
     end do
  endif

  if (massb_time == 2) then ! pas d'interpolation verticale
     bm(:,:) = bm_time(:,:)
     Tann(:,:) = tann_time(:,:)
  else if (massb_time == 3) then ! interpolation verticale
     ! ajuste bm en fonction du temps et du gradient
     bm(:,:) = bm_time(:,:) + grad_smb_time(:,:) * (S(:,:) - S0(:,:))
     tann(:,:)= tann_time(:,:) + grad_tann_time(:,:) * (S(:,:) - S0(:,:))
  endif

end subroutine massb_ISMIP_RCM




end module  climat_InitMIP_years_perturb_mod
